#!/usr/bin/perl

# Try to keep the programming style good ;-)

use strict;
use warnings;

# Some modules we need for password-related Stuff

use Crypt::PasswdMD5;
use Crypt::RandPasswd;

# We want to parse commandline arguments in a comfortable way

use Getopt::Std;

# Parse the commandline arguments and fall back to default values if
# neccessary

my %opts;
getopts('b:g:i:l:s:o:m:x:hr', \%opts);

if ( $opts{h} ) { die "usage: $0 -i [input file] -g [group] -b [base directory for \$HOME] -s [smbpasswd binary] -o [output file] -r -m [minimum password length] -x [maximum password length]\n";}

if ( $> != 0 ) {die "You must be root to run this.\n";}

if ( ! scalar %opts ) {die "usage: $0 -i [input file] -g [group] -b [base directory for \$HOME] -s [smbpasswd binary] -o [output file] -r -m [minimum password length] -x [maximum password length]\n";}

if ( $opts{i} ) { our $userliste = $opts{i}; }
else { our $userliste = "userlist"; }

if ( $opts{g} ) { our $group = $opts{g}; }
else { our $group = "schueler"; }

if ( ! getgrnam $main::group ) { die "Group $main::group doesn't exist. Please create it or specify an existing group.\n" }

if ( $opts{b} ) { our $homebase = $opts{b}; }
else { our $homebase = "/home/$main::group"; }

if ( $opts{s} ) { our $smbpass = $opts{s}; }
else { our $smbpass = "/usr/local/bin/smbpasswd"; }

stat $main::smbpass or die "Couldn't stat $main::smbpass. Please specify\n";

if ( $opts{o} ) { our $outfile = $opts{o}; }
else { our $outfile = "useraccounts"; }

if ( $opts{r} ) { our $passtype = "chars"; }
else { our $passtype = "letters" }

if ( $opts{m} ) { our $minlength = $opts{m}; }
else	{
	if ( $opts{r} ) { our $minlength = 10; }
	else { our $minlength = 8; }
	}

if ( $opts{x} ) { our $maxlength = $opts{x}; }
else	{
	if ( $opts{r} ) { our $maxlength = 10; }
	else { our $maxlength = 8; }
	}

print "Group: $main::group\n";
print "Base directory: $main::homebase\n";
print "Userliste: $main::userliste\n\n";

# Determine the system we run on

if ( ! stat "/usr/bin/uname" ) { our $uname = "/bin/uname"; }
else { our $uname = "/usr/bin/uname" }

my $os = `$main::uname`;
chomp $os;

# Create the filehandle to read the userlist from
open LISTE,$main::userliste or die "$main::userliste: No such file or directory\n";

# Do some deep magic to determine the school year.

# First get the raw time
my @time = localtime;

# Extract the relevant parts

my $month = $time[4];
my $year = $time[5];

# Convert the year to 2-digit format

my $year1 = sprintf("%02d", $year % 100);

# Now it's time for the arcane part...

if ($month le 7) {
	my $year2=$year1 - 1;
	$main::schoolyear = "0$year2/$year1";
}
else {
	my $year2=$year1 + 1;
	$main::schoolyear = "$year1/0$year2";
}

# Extract the account data from the of usernames

while (my $line = <LISTE>)
{
	# Parse the userlist and put it's elements in variables
	my ($name, $realname, $class) = split(/\t/, $line);	

	# Remove the newline left over from parsing the input file

	chomp $class;

	# First check if the username exists. If it exists generate two
	# random letters and attach them to the name. Repeat this
	# procedure if the changed name exists already.

	if ( getpwnam $name ) 
		{
		print "User $name already in passwd database\n";
		unless ( ! getpwnam $name )
			{
			my $randomizer = Crypt::RandPasswd->word(1,2);
			$name .= $randomizer;
			print "New Name: $name\n";
			}
		}
	
	# Compose the GECOS field for /etc/passwd
	my $gecos="$realname;$class;$main::schoolyear";

	# Generate a password for the user

	my $clearpass = Crypt::RandPasswd->$main::passtype( $main::minlength, $main::maxlength );
#	else {our $clearpass = Crypt::RandPasswd->letters( 8, 8 );}

	my $passwdhash = unix_md5_crypt ($main::clearpass, '$1$');

	# Create an Unix account
	# First determine which sytem we run on and which commands we
	# have to use

	if ( $os eq "Linux" )
		{
		# useradd(8) is braindead
		my $home = "$main::homebase/$name";

		system "/usr/sbin/useradd -m -c '$gecos' -g $main::group -d $home $name";
		open PASSPIPE,"| /usr/sbin/chpasswd -e";
		print PASSPIPE "$name:$passwdhash\n";
		close PASSPIPE;
		}

	if ( $os eq "FreeBSD" )
		{
		open PWPIPE, "| /usr/sbin/pw useradd -n $name -m -c '$gecos' -g $main::group -b $main::homebase -h 0";
		print PWPIPE "$main::clearpass\n";
		close PWPIPE;
		}

	if ( $os eq "NetBSD" )
		{
		die "None of the NetBSD user management tools will accept the\npassword on standard input and it would have to be supplied as\ncommandline argument. If you really want to do this\nuncomment line 141 and comment line 140 out I warned you.\n";
#		system "/usr/sbin/useradd -m -c '$gecos' -g $main::group -b $main::homebase -p $passwdhash $name";
		}

	# Use smbpasswd(8) to create an smbpasswd entry
	# Yes this is ugly...and so is smbpasswd's non-interactive
	# mode...

	# Compose the complete smbpasswd command

	my $smbpasswd = "$main::smbpass -s -a $name";

	
	open SMBPIPE,"| $smbpasswd";
	print SMBPIPE "$main::clearpass\n$main::clearpass\n";
	close SMBPIPE;

	# Write the account data to a file

	open OUTFILE,">>$main::outfile";
	print OUTFILE "Username: $name Password: $main::clearpass Name: $realname Klasse: $class\n";
	print "Username: $name Password: $main::clearpass Name: $realname Klasse: $class\n";
	
}


close LISTE;
close OUTFILE;

chmod (0600, $main::outfile);
